﻿using System;
using System.Collections.Generic;
using System.Xml;


/////////////////////////////////////////////////
//      www.interestprograms.ru
//      Программы, игры и их исходные коды
////////////////////////////////////////////////

namespace ASPMVCRouting1.Models
{
    // Здесь находится бизнес-логика приложения


    // Получение данных из базы,
    // в качестве базы выступает xml файл.
    public class GetData
    {
        private XmlDocument _xmlDocument = new XmlDocument();
        private List<Catalog> _listCatalogs = new List<Catalog>();
        private Catalog _catalog = new Catalog();
        private List<Element> _listElements = new List<Element>();
        private Element _element = new Element();
        private List<Item> _listItems = new List<Item>();
        private Item _item = new Item();


        // При создании объекта класса, данные из xml базы 
        // будут загружаться автоматически.
        public GetData()
        {
            _xmlDocument.Load(AppContext.BaseDirectory + "\\DataBase\\database.xml");
        }


        public void PrepareCatalogs()
        {
            XmlNodeList nlist = _xmlDocument.SelectNodes("root/catalog");
            for (int i = 0; i < nlist.Count; i++)
            {
                Catalog catalog = new Catalog();
                catalog.Name = nlist[i].Attributes["name"].Value;
                catalog.Url = nlist[i].Attributes["url"].Value;
                _listCatalogs.Add(catalog);
            }
        }

        // Подготовим данные каталога
        public void PrepareCatalog(string url)
        {
            XmlNode node = _xmlDocument.SelectSingleNode("root/catalog[@url='" + url + "']");
            _catalog.Url = url;
            _catalog.Name = node.Attributes["name"].Value;
            _catalog.Text = node.FirstChild.InnerText;

            // Данные дочерних компонентов
            XmlNodeList nlist = _xmlDocument.SelectNodes("root/catalog[@url='" + url + "']/element");
            List<Element> _listItems = new List<Element>();
            for (int i = 0; i < nlist.Count; i++)
            {
                Element el = new Element();
                el.Name = nlist[i].Attributes["name"].Value;
                el.Url = nlist[i].Attributes["url"].Value;
                _catalog.Elements.Add(el);
            }
        }

        // Подготовим данные запрошенного элемента 
        public void PrepareElement(string urlcatalog, string urlelement)
        {
            XmlNode node = _xmlDocument.SelectSingleNode("/root/catalog[@url='" + urlcatalog + "']/element[@url='" + urlelement + "']");
            _element.Url = urlelement;
            _element.Name = node.Attributes["name"].Value;
            _element.Text = node.SelectSingleNode("./description").InnerText;
            _element.Catalog = urlcatalog;

            // Данные дочерних пунктов
            XmlNodeList nlist = _xmlDocument.SelectNodes("/root/catalog[@url='" + urlcatalog + "']/element[@url='" + urlelement + "']/item");
            for (int i = 0; i < nlist.Count; i++)
            {
                Item item = new Item();
                item.Name = nlist[i].Attributes["name"].Value;
                item.Url = nlist[i].Attributes["url"].Value;
                item.Text = nlist[i].SelectSingleNode("./description").InnerText;
                _element.Items.Add(item);
            }

            // Реквизиты родственных элементов
            XmlNodeList ns = _xmlDocument.SelectNodes("/root/catalog[@url='" + urlcatalog + "']/element");
            for (int i = 0; i < ns.Count; i++)
            {
                Element el = new Element();
                el.Catalog = urlcatalog;
                el.Name = ns[i].Attributes["name"].Value;
                el.Url = ns[i].Attributes["url"].Value;
                _element.Relatives.Add(el);
            }

        }


        // Получение данных запрошенного пункта
        public void PrepareItem(string urlcatalog, string urlelement, string urlitem)
        {
            XmlNode node = _xmlDocument.SelectSingleNode("/root/catalog[@url='" + urlcatalog + "']/element[@url='" + urlelement + "']/item[@url='" + urlitem + "']");
            _item.Catalog = urlcatalog;
            _item.Element = urlelement;
            _item.Url = urlitem;
            _item.Name = node.Attributes["name"].Value;
            _item.Text = node.SelectSingleNode("./description").InnerText;
            XmlNode n = node.SelectSingleNode("./image");
            _item.Image = n.InnerText;

            // Реквизиты родственных пунктов
            XmlNodeList nlist = _xmlDocument.SelectNodes("/root/catalog[@url='" + urlcatalog + "']/element[@url='" + urlelement + "']/item");
            for (int i = 0; i < nlist.Count; i++)
            {
                if (nlist[i].Attributes["url"].Value != _item.Url)
                {
                    Item item = new Item();
                    item.Catalog = urlcatalog;
                    item.Element = urlelement;
                    item.Name = nlist[i].Attributes["name"].Value;
                    item.Url = nlist[i].Attributes["url"].Value;
                    _item.Relatives.Add(item);
                }
            }

        }


        #region Открытые свойства
        public List<Catalog> ListCatalogs
        {
            get { return _listCatalogs; }
        }

        public Catalog Catalog
        {
            get { return _catalog; }
        }

        public List<Element> ListElements
        {
            get { return _listElements; }
        }

        public Element Element
        {
            get { return _element; }
        }

        public Item Item
        {
            get { return _item; }
        }
        #endregion
    }


    #region Классы временного хранения данных

    public class Catalog
    {
        public string Name;
        public string Url;
        public string Text;
        public List<Element> Elements = new List<Element>();
    }

    public class Element
    {
        public string Catalog;
        public string Name;
        public string Url;
        public string Text;
        public List<Item> Items = new List<Item>();

        // Список родственных элементов
        public List<Element> Relatives = new List<Element>();
    }

    public class Item
    {
        public string Catalog;
        public string Element;
        public string Name;
        public string Url;
        public string Text;
        public string Image;

        // Список родственных пунктов
        public List<Item> Relatives = new List<Item>();
    }
    
    #endregion
}